@@ -437,9 +437,11 @@ DocumentPtr DocumentManager::loadDocument(const QString &fileName,
         return mDocuments.at(documentIndex);
 
     // Try to find it in otherwise referenced documents
-    {
-        QString canonicalFilePath = QFileInfo(fileName).canonicalFilePath();
+    QString canonicalFilePath = QFileInfo(fileName).canonicalFilePath();
+    if (!canonicalFilePath.isEmpty()) {
         for (Document *doc : Document::documentInstances()) {
+            if (doc->fileName().isEmpty())
+                continue;
             if (QFileInfo(doc->fileName()).canonicalFilePath() == canonicalFilePath)
                 return doc->sharedFromThis();
         }
@@ -840,7 +842,14 @@ void DocumentManager::tabContextMenuRequested(const QPoint &pos)
 
     QMenu menu(mTabBar->window());
 
-    Utils::addFileManagerActions(menu, mDocuments.at(index)->fileName());
+    const Document *fileDocument = mDocuments.at(index).data();
+    if (fileDocument->type() == Document::TilesetDocumentType) {
+        auto tilesetDocument = static_cast<const TilesetDocument*>(fileDocument);
+        if (tilesetDocument->isEmbedded())
+            fileDocument = tilesetDocument->mapDocuments().first();
+    }
+
+    Utils::addFileManagerActions(menu, fileDocument->fileName());
 
     menu.addSeparator();
 
