@@ -56,6 +56,7 @@ StampBrush::StampBrush(QObject *parent)
     , mIsRandom(false)
     , mIsWangFill(false)
     , mWangSet(nullptr)
+    , mRandomCacheValid(true)
     , mStampActions(new StampActions(this))
 {
     connect(mStampActions->random(), &QAction::toggled, this, &StampBrush::randomChanged);
@@ -215,9 +216,16 @@ void StampBrush::mapDocumentChanged(MapDocument *oldDocument,
 {
     AbstractTileTool::mapDocumentChanged(oldDocument, newDocument);
 
+    if (oldDocument) {
+        disconnect(oldDocument, &MapDocument::tileProbabilityChanged,
+                   this, &StampBrush::invalidateRandomCache);
+    }
+
     if (newDocument) {
-        updateRandomList();
+        invalidateRandomCache();
         updatePreview();
+        connect(newDocument, &MapDocument::tileProbabilityChanged,
+                this, &StampBrush::invalidateRandomCache);
     }
 }
 
@@ -264,7 +272,7 @@ void StampBrush::setStamp(const TileStamp &stamp)
 
     mStamp = stamp;
 
-    updateRandomList();
+    invalidateRandomCache();
     updatePreview();
 }
 
@@ -379,6 +387,11 @@ void StampBrush::drawPreviewLayer(const QVector<QPoint> &points)
         return;
 
     if (mIsRandom) {
+        if (!mRandomCacheValid) {
+            updateRandomList();
+            mRandomCacheValid = true;
+        }
+
         if (mRandomCellPicker.isEmpty())
             return;
 
@@ -605,7 +618,7 @@ void StampBrush::setRandom(bool value)
         mStampActions->wangFill()->setChecked(false);
     }
 
-    updateRandomList();
+    invalidateRandomCache();
     updatePreview();
 }
 
@@ -623,3 +636,8 @@ void StampBrush::setWangFill(bool value)
 
     updatePreview();
 }
+
+void StampBrush::invalidateRandomCache()
+{
+    mRandomCacheValid = false;
+}
